#! ver 1.00.00
#--
# Credit: Dan Rathbun, Palm Bay, FL - danzoid61@gmail.com
# Date  : 23 NOV 2009
# Terms : Free Use - Public Domain
# -------------------------------------------------------------------
# Disclaimer:
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
# -------------------------------------------------------------------
#
# class Integer add-in for RGB, RGB0, RGBA and ARGB conversion
# 	subclasses Fixnum and Bignum will inherit these methods.
#
# NOTE! Ruby has no way of knowing whether a given integer is meant
#		to be an ARGB or RGBA color integer. It's up to ruby programmers
#		to apply the use of these methods properly.
#
#++

class ::Integer


	#######################################################
	#
	#== _int_.to_ARGB_i -> Integer (Fixnum or Bignum)
	#
	# Convert an RGB, RGB0 or RGBA color Integer (from OpenGL,)
	#  into an ARGB color Integer, for use with, HTML, CSS,
	#  Microsoft .NET, or the Windows GDI+ library.
	#
	# RGB and RGB0 color integers have the alpha (4th) byte set to 00.
	#  These are compatible with the 'old' Windows GDI library.
	#
	# RGBA color integers can have an alpha value from 00..FF (0..255).
	#
	# Returns:
	#  integer:: a 32 bit ARGB integer if successful
	#
	# Useage: Given an OpenGL or Sketchup::Color integer = 0x8099CC38,
	#  which is a color with components: [R:0x38,G:0xCC,B:0x99,A:0x80],
	#  Assume variable rgbaColor = 0x8099CC38, to convert it to ARGB:
	#  argbColor = rgbaColor.to_ARGB_i
	#  returns:
	#  argbColor = 0x8038CC99
	#
	# Ignores: any numerical amount above 32bits
	#
	def to_ARGB_i		# :doc:
		#--
		#	Reminder:
		#		ARGB byte order, listed MSB to LSB:
		#   ( 0x AA RR GG BB ) Hexidecimal notation
		#   Binary order, bits listed normally, MSB to LSB:
		#   ( aaaaaaaa rrrrrrrr gggggggg bbbbbbbb )
		#
		# We need to swap the red and blue bytes
		#
		i = self.abs  # get rid of any negative sign
		#
		# first 8 bit byte is red in RGBA, save it
		r = 0xFF & i  # only copy 1st 8 set bits from i
		r = r << 16   # left shift 16 bits, it's 3rd in ARGB
		#
		# second 8 bit byte is green in both ARGB and RGBA
		# green is in the right place, just save it
		g = 0xFF00 & i  # only copy 2nd 8 set bits from i
		#
		# third 8 bit byte is blue in RGBA, save it
		b = 0xFF0000 & i  # only copy 3rd 8 set bits from i
		b = b >> 16       # right shift 16 bits, it's first in ARGB
		#
		# fourth 8 bit byte is alpha in both ARGB and RGBA
		# alpha is in the right place,  
		a = 0xFF000000 & i  # only copy 4th 8 set bits from i
		#
		# sum and return using bitwise OR
		return ( a | r | g | b )
		#++
	end


	#######################################################
	#
	#== _int_.to_RGB0_i -> Integer (Fixnum or Bignum)
	#
	# alias for Fixnum.to_RGB_i
	#
	# Convert an ARGB Integer Fixnum (ie, a standard ruby integer,)
	#  into a RGB0 Integer, for use with OpenGL and Sketchup.
	#  Also compatible with the 'old' Microsoft GDI library.
	#  RGB0 integers always have the alpha value [4th byte] set 00.
	#  They are then basically the same as 24bit RGB color integers.
	#
	# Returns:
	#  integer:: a 32 bit RGB0 integer if successful
	#
	# Useage: Sketchup::Color.new( 0x99CC38.to_RGB0_i )
	#  creates a color: [R:0x38,G:0xCC,B:0x99,A:0x00]
	#
	# Ignores: any numerical amount above 32bits
	#
	def to_RGB0_i		# :doc:
		#--
		# Reminder:
		#		RGB0 byte order, listed MSB to LSB:
		#   ( 0x 00 BB GG RR ) Hexidecimal notation
		#   Binary order, bits listed normally, MSB to LSB:
		#   ( 00000000 bbbbbbbb gggggggg rrrrrrrr )
		#
		# We need to swap the red and blue bytes, set alpha 0
		#
		i = self.abs  # get rid of any negative sign
		# first 8 bit byte is blue in ARGB, save it
		b = 0xFF & i  # only copy 1st 8 set bits from i
		b = b << 16   # left shift 16 bits, it's 3rd in RGBA
		#
		# second 8 bit byte is green in both ARGB and RGBA
		# green is in the right place, just save it
		g = 0xFF00 & i  # only copy 2nd 8 set bits from i
		#
		# third 8 bit byte is red in ARGB, save it
		r = 0xFF0000 & i  # only copy 3rd 8 set bits from i
		r = r >> 16  # right shift 16 bits, it's first in RGBA
		#
		# fourth 8 bit byte is alpha in both ARGB and RGBA
		# alpha is in the right place,  
		a = 0x00000000  # set alpha byte to zero !
		# sum and return using bitwise OR
		return ( r | g | b | a )
		#++
	end
	alias_method(:to_RGB_i, :to_RGB0_i)


	#######################################################
	#
	#== _int_.to_RGBA_i -> Integer (Fixnum or Bignum)
	#
	# Convert an ARGB Integer Fixnum (ie, an HTML color integer,)
	#  into a RGBA Integer, for use with OpenGL and Sketchup.
	#  RGBA integers hold a vaild alpha value in the 4th byte.
	#
	# Returns:
	#  integer:: a 32 bit RGBA integer if successful
	#
	# Useage: Sketchup::Color.new( 0x8099CC38.to_RGBA_i )
	#  creates a half-transparent color: [R:0x38,G:0xCC,B:0x99,A:0x80]
	#
	# Ignores: any numerical amount above 32bits
	#
	def to_RGBA_i		# :doc:
		#-- 
		# Reminder:
		#		RGBA byte order, listed MSB to LSB:
		#   ( 0x AA BB GG RR ) Hexidecimal notation
		#   Binary order, bits listed normally, MSB to LSB:
		#   ( aaaaaaaa bbbbbbbb gggggggg rrrrrrrr )
		#
		# We need to swap the red and blue bytes
		#
		i = self.abs  # get rid of any negative sign
		#
		# first 8 bit byte is blue in ARGB, save it
		b = 0xFF & i  # only copy 1st 8 set bits from i
		b = b << 16   # left shift 16 bits, it's 3rd in RGBA
		#
		# second 8 bit byte is green in both ARGB and RGBA
		# green is in the right place, just save it
		g = 0xFF00 & i  # only copy 2nd 8 set bits from i
		#
		# third 8 bit byte is red in ARGB, save it
		r = 0xFF0000 & i  # only copy 3rd 8 set bits from i
		r = r >> 16       # right shift 16 bits, it's first in RGBA
		#
		# fourth 8 bit byte is alpha in both ARGB and RGBA
		# alpha is in the right place, just save it
		a = 0xFF000000 & i  # only copy 4th 8 set bits from i
		#
		# sum and return using bitwise OR
		return ( r | g | b | a )
		#++
	end
	
	
end # class Integer add-in